         LIST P=16C54
;************************************************************ 
;       64K Page Write Program (126 bytes)
;
;       The 24LC65 has a page length of 8 bytes.  This page
;       can be used to write up to 8 bytes of data into the
;       part before initiating the write cycle.  Since the
;       write cycle is timed the same for 1 byte or 8 bytes
;       it is more efficient to use the page mode when
;       consecutive addresses are being written to.
;
;       When using page mode, the control byte, upper and lower
;       addresses are sent for the first address only. After the  
;       data byte for the first address is sent, the data for the
;       next consecutive address is clocked in.  This is 
;       repeated as many times as needed (as long as the page
;       length is not exceeded) and then a stop bit is sent.
;       The device will still acknowledge between every byte of
;       data.  After the stop bit is sent, the part will 
;       initiate the self timed write cycle.  
;
;       This routine waits approximately 10mS for the write 
;       cycle to complete for each page.  A more efficient
;       method of determining when the write cycle is complete
;       is called "data polling."  The data polling method is
;       explained in the program "64kdpoll.asm."  That 
;       particular program uses data polling in a byte write
;       mode but it can be used in exactly the same way for
;       a page mode write.    
;
;       As an option, the user can connect a LED to pin 18
;       on the PIC (use about a 1K resistor in series) as a 
;       acknowledge fail indicator.  This LED will come on 
;       if the device being programmed does not send a low
;       acknowledge bit at the proper times.  This can be
;       tested by simply running the program with no part
;       in the socket.
;
;       Timing is based on using the PIC in 'XT' mode
;       using a 4Mhz crystal.  Clock speeds to the serial EE 
;       will be approximately 40 Khz for this setup.
;
;
;        PIC to Serial EE Connections:
;
;           PIC          Serial EE
;       ------------    ------------
;       Pin 12 (RB6) -->  SCLK
;       Pin 13 (RB7) -->  SDATA
;
;       PIN 18 (RA1) -->  Acknowledge fail LED  (Optional)
;
;
;************************************************************
;           Register Definitions
;************************************************************
port_a  equ     5h     ; port 5 (port_a) used for LEDs
port_b  equ     6h     ; port 6 (port b) used for data and 
                       ; clock lines
eeprom  equ    0ah     ; bit buffer
bycnt   equ    0bh     ; byte counter for read mode
addr    equ    0ch     ; word 0 address counter
datai   equ    0dh     ; data input register
datao   equ    0eh     ; data output register
slave   equ    0fh     ; device address (1010xxx0)
txbuf   equ    10h     ; transmit buffer
count   equ    11h     ; bit counter
bcount  equ    12h     ; byte counter
loops   equ    15h     ; delay loop counter
loops2  equ    16h     ; delay loop counter
addr1   equ    17h     ; word 1 address counter
;************************************************************
;           Bit Definitions
;************************************************************
di      equ    7       ; eeprom input bit
do      equ    6       ; eeprom output bit
sdata   equ    7       ; serial EE data line (port_b,pin 13)
sclk    equ    6       ; serail EE clock line (port_b,pin 12)
ackf    equ    1       ; acknowledge fail LED (port_a,pin 18)
;************************************************************
;
;
        org    01ffh   ; set reset vector
        goto   PWRUP
        org    000h    ; 
        goto   PWRUP  
;
;*************************************************************
;       DELAY ROUTINE
;       This routine takes the value in 'loops'
;       and multiplies it times 1 millisecond to
;       determine delay time.  
;*************************************************************
WAIT
;
top      movlw   .110           ; timing adjustment variable
         movwf   loops2
top2     nop                    ; sit and wait
         nop
         nop
         nop
         nop
         nop
         decfsz  loops2         ; inner loops complete?
         goto    top2           ; no, go again
                                ;
         decfsz  loops          ; outer loops complete?
         goto    top            ; no, go again
         retlw   0              ; yes, return from sub
;
;**************************************************************
;       Start Bit Subroutine
;       this routine generates a start bit
;       (Low going data line while clock is high)          
;**************************************************************
;
BSTART
         bsf     port_b,sdata   ; make sure data is high
         movlw   b'00111111'
         tris    port_b         ; set data and clock lines for output
         bcf     port_b,sclk    ; make sure clock is low
         nop
         bsf     port_b,sclk    ; set clock high
         nop
         nop
         nop
         nop
         nop
         bcf     port_b,sdata   ; data line goes low during
                                ; high clock for start bit
         nop
         nop
         nop
         nop
         nop                    ; timing adjustment
         bcf     port_b,sclk    ; start clock train
         nop
         nop
         retlw   0
;        
;************************************************************
;       Stop Bit Subroutine
;       This routine generates a stop bit
;       (High going data line while clock is high)
;************************************************************
BSTOP
         movlw   b'00111111'    ;
         tris    port_b         ; set data/clock lines as outputs
         bcf     port_b,sdata   ; make sure data line is low
         nop
         nop
         nop
         bsf     port_b,sclk    ; set clock high
         nop
         nop    
         nop
         bsf     port_b,sdata   ; data goes high while clock high
                                ; for stop bit
         nop
         nop
         bcf     port_b,sclk    ; set clock low again
         nop
         nop
         nop
         retlw   0
;
;*************************************************************
;       BITOUT routine takes one bit of data in 'do' and
;       transmits it to the serial EE device
;*************************************************************
BITOUT
         movlw   b'00111111'    ; set data,clock as outputs
         tris    port_b
         btfss   eeprom,do      ; check for state of data bit to xmit
         goto    bitlow         ; 
         bsf     port_b,sdata   ; set data line high 
         goto    clkout         ; go toggle the clock

bitlow   bcf     port_b,sdata   ; output a low bit     
clkout   bsf     port_b,sclk    ; set clock line high
         nop
         nop
         nop
         nop
         bcf     port_b,sclk    ; return clock line low
         retlw   0
;
;**************************************************************
;       BITIN routine reads one bit of data from the 
;       serial EE device and stores it in 'di'
;**************************************************************
BITIN
         bsf     eeprom,di      ; assume input bit is high
         movlw   b'10111111'    ; make sdata an input line
         tris    port_b
         bsf     port_b,sdata   ; set sdata line for input
         bsf     port_b,sclk    ; set clock line high
         nop                    ; just sit here a sec
         nop
         nop
         nop
         nop                    ;
         btfss   port_b,sdata   ; read the data bit
         bcf     eeprom,di      ; input bit was low
         bcf     port_b,sclk    ; set clock line low
                                ;
         retlw   0              ; 
;
;****************************************************************
;       Transmit Data Subroutine
;       This routine takes the byte of data stored in the
;       'datao' register and transmits it to the serial EE device.
;       It will then send 1 more clock to the serial EE for the
;       acknowledge bit.  If the ack bit from the part was low
;       then the transmission was sucessful.  If it is high, then
;       the device did not send a proper ack bit and the ack
;       fail LED will be turned on.
;****************************************************************
TX
         movlw   .8
         movwf   count          ; set the #bits to 8
         ;
TXLP
         bcf     eeprom,do      ; assume bit out is low
         btfsc   txbuf,7        ; is bit out really low?
         bsf     eeprom,do      ; otherwise data bit =1
         call    BITOUT         ; serial data out
         rlf     txbuf          ; rotate txbuf left
         decfsz  count          ; 8 bits done?
         goto    TXLP           ; no - go again
         call    BITIN          ; read ack bit
         btfsc   eeprom,di      ; check ack bit
         bsf     port_a,ackf    ; set acknowledge fail LED if the
         ;
         retlw   0
;    
;****************************************************************
;       Power up routine
;       This is the program entry point, which in this case simply
;       sets the port_a I/O lines and directs control to the
;       byte write routine.
;****************************************************************
PWRUP
          movlw  b'00000000'
          tris   port_a         ; set port A as all output
          clrf   port_a         ; all output lines low
          goto   pwrite         ; go do the page write

;****************************************************************
;       Page Write Routine
;       This routine writes the data in "datao" to 8 consecutive 
;       bytes using page write mode starting at address 00. 
;       This routine waits 10mS after every page to give the device 
;       time to do the write.  This program repeats forever.  
;*****************************************************************
;
pwrite
;                            
         clrf    port_a      ; clear all LEDs
         movlw   b'10100000' ; set slave address and write mode
         movwf   slave
         movlw   b'01010101' ; set data to write as 55h
         movwf   datao
;        
         movlw   .8          ; set number of bytes
         movwf   bcount      ; to write as to 8    
         clrf    addr1       ; set high address byte to 00 
         clrf    addr        ; set low address byte to 00 
                             ;
         call    BSTART      ; generate start bit
         movf    slave,w     ; move slave address
         movwf   txbuf       ; into transmit buffer
         call    TX          ; and send it        
         movf    addr1,w     ; move word 1 address
         movwf   txbuf       ; into transmit buffer
         call    TX          ; and send it        
         movf    addr,w      ; move word 0 address
         movwf   txbuf       ; into transmit buffer
         call    TX          ; and send it        

byte     movf    datao,w     ; move data byte
         movwf   txbuf       ; to transmit buffer
         call    TX          ; and transmit it
         decfsz  bcount      ; all 8 bytes written?
         goto    byte        ; no, do another byte
                             ; 
         call    BSTOP       ; generate stop bit
         movlw   .10
         movwf   loops       ; set delay time to give
         call    WAIT        ; 10 ms wait after every byte
         goto    pwrite      ; start over
;
;
END              
         


