         LIST P=16C54
;************************************************************ 
;       Program to set the high endurance block on the 64K
;       device (24LC65).     (122 bytes)
;
;       The 24LC65 is a 64K device, 4K of which is error
;       corrected to increase the endurance of that portion
;       of the array.  The location of this high endurance
;       'block' can be set by the user.  When the device
;       comes from the factory, the high endurance block
;       will be set as the uppermost block in the array.
; 
;       This program sets the high endurance block as the
;       first block in the array (address range 00h to 1FFh).
;
;       The high endurance block is set by sending the following
;       sequence to the device:
;
;       SB 10100000 1XXAAAAX XXXXXXXX 00XX0000 STB
;
;       Where SB=start bit
;              1=data high
;              0=data low
;              X=don't care
;           AAAA=4 bit high endurance block address
;            STB=stop bit
; 
;       As an option, the user may connect a LED to pin 18
;       on the PIC (use about a 1K resistor in series) as an
;       acknowledge fail indicator.  This LED will come on if
;       the serial EE fails to acknowledge correctly.  
;
;       Timing is based on using the PIC in 'XT' mode
;       using a 4Mhz crystal.  Clock speeds to the serial EE 
;       will be approximately 60 Khz for this setup.
;
;        PIC to Serial EE Connections:
;
;           PIC          Serial EE
;       ------------    ------------
;       Pin 12 (RB6) -->  SCLK
;       Pin 13 (RB7) -->  SDATA
;
;       PIN 18 (RA1) -->  Acknowledge fail LED  (Optional);
;************************************************************
;           Register Definitions
;************************************************************
port_a  equ     5h     ; port 5 (port_a) used for LEDs
port_b  equ     6h     ; port 6 (port b) used for data and 
                       ; clock lines
eeprom  equ    0ah     ; bit buffer
bycnt   equ    0bh     ; byte counter for read mode
addr    equ    0ch     ; word 0 address counter
datai   equ    0dh     ; data input register
datao   equ    0eh     ; data output register
slave   equ    0fh     ; device address (1010xxx0)
txbuf   equ    10h     ; transmit buffer
count   equ    11h     ; bit counter
bcount  equ    12h     ; byte counter
loops   equ    15h     ; delay loop counter
loops2  equ    16h     ; delay loop counter
addr1   equ    17h     ; word 1 address counter
he_blk  equ    18h     ; high endurance block address 
;************************************************************
;           Bit Definitions
;************************************************************
di      equ    7       ; eeprom input bit
do      equ    6       ; eeprom output bit
sdata   equ    7       ; serial EE data line (port_b,pin 13)
sclk    equ    6       ; serail EE clock line (port_b,pin 12)
ackf    equ    1       ; acknowledge fail LED (port_a,pin 18)
;************************************************************
;
;
        org    01ffh   ; set reset vector
        goto   PWRUP
        org    000h    ; 
        goto   PWRUP  
;
;*************************************************************
;       DELAY ROUTINE
;       This routine takes the value in 'loops'
;       and multiplies it times 1 millisecond to
;       determine delay time.  
;*************************************************************
WAIT
;
top      movlw   .110           ; timing adjustment variable
         movwf   loops2
top2     nop                    ; sit and wait
         nop
         nop
         nop
         nop
         nop
         decfsz  loops2         ; inner loops complete?
         goto    top2           ; no, go again
                                ;
         decfsz  loops          ; outer loops complete?
         goto    top            ; no, go again
         retlw   0              ; yes, return from sub
;
;**************************************************************
;       Start Bit Subroutine
;       this routine generates a start bit
;       (Low going data line while clock is high)          
;**************************************************************
;
BSTART
         bsf     port_b,sdata   ; make sure data is high
         movlw   b'00111111'
         tris    port_b         ; set data and clock lines for output
         bcf     port_b,sclk    ; make sure clock is low
         nop
         bsf     port_b,sclk    ; set clock high
         nop
         nop
         nop
         nop
         nop
         bcf     port_b,sdata   ; data line goes low during
                                ; high clock for start bit
         nop
         nop
         nop
         nop
         nop                    ; timing adjustment
         bcf     port_b,sclk    ; start clock train
         nop
         nop
         retlw   0
;        
;************************************************************
;       Stop Bit Subroutine
;       This routine generates a stop bit
;       (High going data line while clock is high)
;************************************************************
BSTOP
         movlw   b'00111111'    ;
         tris    port_b         ; set data/clock lines as outputs
         bcf     port_b,sdata   ; make sure data line is low
         nop
         nop
         nop
         bsf     port_b,sclk    ; set clock high
         nop
         nop    
         nop
         bsf     port_b,sdata   ; data goes high while clock high
                                ; for stop bit
         nop
         nop
         bcf     port_b,sclk    ; set clock low again
         nop
         nop
         nop
         retlw   0
;
;*************************************************************
;       BITOUT routine takes one bit of data in 'do' and
;       transmits it to the serial EE device
;*************************************************************
BITOUT
         movlw   b'00111111'    ; set data,clock as outputs
         tris    port_b
         btfss   eeprom,do      ; check for state of data bit to xmit
         goto    bitlow         ; 
         bsf     port_b,sdata   ; set data line high 
         goto    clkout         ; go toggle the clock

bitlow   bcf     port_b,sdata   ; output a low bit     
clkout   bsf     port_b,sclk    ; set clock line high
         nop
         nop
         nop
         nop
         bcf     port_b,sclk    ; return clock line low
         retlw   0
;
;**************************************************************
;       BITIN routine reads one bit of data from the 
;       serial EE device and stores it in 'di'
;**************************************************************
BITIN
         bsf     eeprom,di      ; assume input bit is high
         movlw   b'10111111'    ; make sdata an input line
         tris    port_b
         bsf     port_b,sdata   ; set sdata line for input
         bsf     port_b,sclk    ; set clock line high
         nop                    ; just sit here a sec
         nop
         nop
         nop
         nop                    ;
         btfss   port_b,sdata   ; read the data bit
         bcf     eeprom,di      ; input bit was low
         bcf     port_b,sclk    ; set clock line low
                                ;
         retlw   0              ; 
;
;****************************************************************
;       Transmit Data Subroutine
;       This routine takes the byte of data stored in the
;       'datao' register and transmits it to the serial EE device.
;       It will then send 1 more clock to the serial EE for the
;       acknowledge bit.  If the ack bit from the part was low
;       then the transmission was sucessful.  If it is high, then
;       the device did not send a proper ack bit and the ack
;       fail LED will be turned on.
;****************************************************************
TX
         movlw   .8
         movwf   count          ; set the #bits to 8
         ;
TXLP
         bcf     eeprom,do      ; assume bit out is low
         btfsc   txbuf,7        ; is bit out really low?
         bsf     eeprom,do      ; otherwise data bit =1
         call    BITOUT         ; serial data out
         rlf     txbuf          ; rotate txbuf left
         decfsz  count          ; 8 bits done?
         goto    TXLP           ; no - go again
         call    BITIN          ; read ack bit
         btfsc   eeprom,di      ; check ack bit
         bsf     port_a,ackf    ; set acknowledge fail LED if the
         ;
         retlw   0
;    
;****************************************************************
;       Power up routine
;       This is the program entry point, which in this case simply
;       sets the port_a I/O lines and directs control to the
;       byte write routine.
;****************************************************************
PWRUP
          movlw  b'00000000'
          tris   port_a         ; set port A as all output
          clrf   port_a         ; all output lines low
          goto   setheblk       ; set go set the high endurance block

;****************************************************************
;       Set High Endurance Block Routine
;       This routine sets the high endurance block to the first
;       block in the array and then delays about a half second.  This
;       routine will repeat forever.
;*****************************************************************
;
setheblk
;                            
        clrf    port_a          ; clear all LEDs
        movwf   slave
        movlw   .0
        movwf   he_blk          ; set the endurance block as first
                                ; block in array (block 0)
        rlf    he_blk           ; rotate starting block left 1 bit
                                ; to arrange it correctly
        bsf     he_blk,7        ; set msb bit in block address byte 
        call    BSTART          ; generate start bit
        movlw   b'10100000'     ; get slave address and write mode
        movwf   txbuf           ; into transmit buffer                
        call    TX              ; and send it
        movf    he_blk,w        ; get the block address byte 
        movwf   txbuf           ; into transmit buffer                
        call    TX              ; and send it
        clrf    txbuf           ; clear buffer
        call    TX              ; send 8 don't care bits
        clrf    txbuf
        call    TX              ; send config byte(all 0's)
        call    BSTOP           ; send stop bit


        movlw   .255            ; long delay
        movwf   loops
        call    wait
        movlw   .255
        movwf   loops
        call    wait

        goto    setheblk        ; go again
;
;
END              
         


